// Copyright (C) 2006 The Ascal Authors - see AUTHORS for more information
//  
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software 
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#include "board.hpp"
//#define DEBUG
#include "ourmove.hpp"
#include "helper.h"
#include "math.h"


board::board()
{
	clear();	
}

void board::clear() {
	int i;
	for(i=0; i <= 10; i++)
	{
		game[i].anz_oben = 1;
		game[i].off_oben = 0;
		game[i].anz_unten = 0;
		game[i].off_unten = 0;
		game[i].number=i;
	}
	
	for(i = 11; i <= 13; i++)
	{
		game[i].anz_oben = 0;
		game[i].off_oben = 0;
		game[i].anz_unten = 0;
		game[i].off_unten = 0;
		game[i].number=i;	
	}

	for(i = 14; i <= 24; i++)
	{
		game[i].anz_oben = -1;
		game[i].off_oben = 0;
		game[i].anz_unten = 0;
		game[i].off_unten = 0;
		game[i].number=i;
	}
}

board::~board()
{
}

fieldvec_t board::getBoard()
{
	fieldvec_t foo;
	for(int i=0;i<25;i++) {
		foo.push_back(game[i]);
	}
	return foo;
}

void board::setBoard(fieldvec_t foo) {
	for(int i=0;i<25;i++) {
		game[i]=foo[i];
	}
	//tMove.clear(); //FIXMEmoh is this necessary?
}

field_t board::getField(int nr)
{
	return game[nr];
}

move_signal board::move(ourmove_t train, bool player)
{
	move_signal flag=check_move(train,player);
	if(flag!=NORMAL && flag!=NORMAL_OFFICER) {
		return flag;
	}

	//and now we know that all is right and play that thing into the real board
	for(int i=0;i<(int)tMove.size();i++) {
		debug("saving field nr " << tMove[i].number);
		game[tMove[i].number]=tMove[i];
	}
	tMove.clear();

	return flag;
}


bool board::out_of_bounds(int pos, int step) {
		if(pos+step<0||pos+step>24) {
			return true;
		}

		if ((step == 8 || step == -6) && pos % 7 == 6) {		//diese 4 if's stellen sicher, dass der stein nicht über den feldrand gesetzt wird
			//debug("Don't go out of the field");
			return true;
		}
	
		if ((step == -4 || step == 3 || step == 6 || step == -8) && pos % 7 == 0) {
			//debug("Don't go out of the field");
			return true;
		} 
	
		if ((step == -3 || step == 4 || step == -6 || step == 8) && pos % 7 == 3) {
			//debug("Don't go out of the field");
			return true;
		} 
	
		if ((step == 6 || step == -8) && pos % 7 == 4) {
			//debug("Don't go out of the field");
			return true;
		}
		return false;
}

move_signal board::check_move(ourmove_t train, bool player)
{
	if(train.size()<2) {
		debug("move not stay");
		return MOVE;
	} //zielfeld darf nicht startfeld sein
	
	tMove.clear();
	
	int k;
	//crazy conversion 'cause of bad arrangement
	if(!player) k=-1;
	else k=1;

	int i=0, step=0;
	field_t tField,fField,sField;
	for(;i<(int)train.size()-1;i++) {
		if(train[i]<0 || train[i]>24) {						//for the crazy ai
			debug("Field out of range");
			return WRONG;
		}
		if(getEField(train[i]).anz_oben*k<=0) {		//gehoert startfeld dem ziehenden spieler?
			debug("Player and move don't fit");

			return WRONG_STONE;
		} 
		
		if(getEField(train[i+1]).anz_oben != 0) {			// ist das letzte zielfeld belegt
			debug("nicht erlaubtes zielfeld");

			return FIELD_FULL;
		}


		//now the direction and size of the move
		step=train[i+1]-train[i];
		if(i==0 &&  !(getEField(train[i]).off_oben%2) &&
		!((step == k*3 || step == k*4 || step == k*6 || step == k*8))) {
			debug("First move without office not allowed");
			return WRONG_DIRECTION;									//zug ohne officer nur am anfang
		}

		if(i!=0 &&  !(getEField(train[i]).off_oben%2) &&
		!((step == k*6 || step == k*8))) {						//ohne office nach dem 1. mal
			debug("Following move without office not allowed");
			return WRONG_DIRECTION;
		}
		if(i==0 &&  (getEField(train[i]).off_oben%2) &&
		!((abs(step) == 3 || abs(step) == 4 || abs(step) == 6 || abs(step) == 8))) {
			debug("First move with office not allowed");
			return WRONG_DIRECTION;									//zug mit officer nur am anfang
		}

		if(i!=0 &&  (getEField(train[i]).off_oben%2) &&
		!((abs(step) == 6 || abs(step) == 8))) {				//mit office nach dem 1. mal
			debug("Following move with office not allowed");
			return WRONG_DIRECTION;
		}

		if(i==0 && (abs(train[i+1]-train[i])<6) &&
			is_there_a_forced_move(-1,k)				//besteht zugzwang am anfang
		) {
			debug("First round and you have to strike");
			return SINGLE;
		}

		if(out_of_bounds(train[i],step)) {	//look whether the move would lead out of the boundaries
			debug("Out of Bounds");
			return WRONG;
		}

		for(int j=0;j<i;j++) {				//the rules say that a field should not
											//be playable twice in 1 move
			if(train[j]==train[i]) {			//FIXMEmoh - it has to be more efficient
				debug("this field is a 2nd time in the move");		//it's not necessary to check _all_ previous fields
				return FIELD_DOUBLE;
			}
		}
		
		sField=getEField(train[i]);
		fField=getEField(train[i+1]);
		
		fField.anz_oben=sField.anz_oben;
		fField.anz_unten=sField.anz_unten;
		fField.off_oben=sField.off_oben;
		fField.off_unten=sField.off_unten;
		
		sField.anz_oben=0;
		sField.anz_unten=0;
		sField.off_oben=0;
		sField.off_unten=0;
		

		if(abs(step)>5) {									//routine for strinking

			if(getEField(train[i]+(train[i+1]-train[i])/2).anz_oben *k >= 0 ) {
											//is the the same color or void between?
				debug("Striking: same color or void in the way");
				return CANT_BEAT;
			}
			tField=getEField(train[i]+ step/2);
			tField.anz_oben=tField.anz_oben+k;		//delete one from the top

			fField.anz_unten=fField.anz_unten - k;	//add one to the cellar
			
			if(tField.off_oben%2){
				fField.off_unten = fField.off_unten | 1<<(abs(fField.anz_unten)-1);
			}
			
			tField.off_oben=tField.off_oben>>1;		//remove the old officer

			if(tField.anz_oben==0) {				//roof empty - swap cellar2roof
				tField.anz_oben=tField.anz_unten;
				tField.off_oben=tField.off_unten;
				tField.anz_unten=0;
				tField.off_unten=0;
			}
			tMove.push_back( tField );			
		}
		
		tMove.push_back(sField);
		tMove.push_back(fField);
	
	}
	debug("Now we are behind the long loop");
	if(abs(step)>5 && is_there_a_forced_move(train.back(),k)) {
		debug("Sorry - you should strike");
		return DOUBLE;
	}

	if( ( (train.back()<25 && train.back()>20 && k == 1) ||
		  (train.back()>-1 && train.back()<4 && k == -1) ) &&
	 	!(getEField(train.back()).off_oben % 2) )
												//build officer
	{
			debug("Building Officer");
			tField=getEField(train.back());
			tField.off_oben++;
			tMove.push_back(tField);	
			return NORMAL_OFFICER;	
	}

	return NORMAL;
}	

bool board::victory(bool l)
{
	int k;
	if(!l) {  k=-1; }
	else { k=1; }
	 
	for(int i = 0; i <25; i++)
	{
		if(game[i].anz_oben*k > 0)
		{
			if(!out_of_bounds(i, k*3)&& !game[i+k*3].anz_oben) return false;
			if(!out_of_bounds(i, k*4)&& !game[i+k*4].anz_oben) return false;
			if(	!out_of_bounds(i, k*6)&& !game[i+k*6].anz_oben
				&& game[i+k*3].anz_oben*k<0)
					return false;
			if(	!out_of_bounds(i, k*8)&& !game[i+k*8].anz_oben
				&& game[i+k*4].anz_oben*k<0)
					return false;
			//debug("Der halbe Sieg");
			if(game[i].off_oben % 2)
			{
				if(!out_of_bounds(i, -(k*3))&& !game[i-k*3].anz_oben) return false;
				if(!out_of_bounds(i, -(k*4))&& !game[i-k*4].anz_oben) return false;
				if(	!out_of_bounds(i, -(k*6))&& !game[i-k*6].anz_oben
					&& game[i-k*3].anz_oben*k<0)
						return false;
				if(	!out_of_bounds(i, -(k*8))&& !game[i-k*8].anz_oben
					&& game[i-k*4].anz_oben*k<0)
						return false;
			}	
		}
	}	  
debug("SIEG!!!");
	return true;
	
}

bool board::is_there_a_forced_move(int field,int k)
{
	//debug("looking for forced moves on field " << field);
	//k is our player
	
	field_t tField;
	if(field==-1) { 
		
		for (int i = 0; i < 25; i++) {	
			tField=getEField(i);
			if(tField.anz_oben*k>0) {
				if(is_there_a_forced_move(i,k))
					return true;	
			}
		}
			
	}
	else {
		
		if(	(!in_temp(field+k*6) && !out_of_bounds(field,k*6) && getEField(k*6+field).anz_oben==0
			&&  getEField(k*3+field).anz_oben*k<0) ||
			(!in_temp(field+k*8) && !out_of_bounds(field,k*8) && getEField(k*8+field).anz_oben==0
			&&  getEField(k*4+field).anz_oben*k<0)
		) {
			debug("Without officer forcing ruled you out");
			return true;		
		}
		tField=getEField(field);
		if(	(tField.off_oben && (tField.off_oben%2)) &&
			((!in_temp(field-k*6) && !out_of_bounds(field,-k*6) && getEField(-k*6+field).anz_oben==0
			&&  getEField(-k*3+field).anz_oben*k<0) ||
			(!in_temp(field-k*8) && !out_of_bounds(field,-k*8) && getEField(-k*8+field).anz_oben==0
			&&  getEField(-k*4+field).anz_oben*k<0))
		) {
			debug("With officer forcing ruled you out");
			return true;
		}
	
	}
	return false;
}

ourmove_t board::goback(ourmove_t train) {
	field_t sField={},fField={},tField={};
	
	if(train.back()==-1) {
		train.pop_back();
		sField=getField(train.back());
		sField.off_oben--;
		game[sField.number]=sField;		
	}
	
	int k,step;
	
	for(int i=train.size()-1;i>0;i--) {
				
		sField=getField(train[i]);
		fField=getField(train[i-1]);
		step=train[i-1]-train[i];
		
		if(sField.anz_oben>0) {
			k=1;
		}
		else
			k=-1;
		
		fField.anz_oben=sField.anz_oben;
		fField.anz_unten=sField.anz_unten;
		fField.off_oben=sField.off_oben;
		fField.off_unten=sField.off_unten;
		
		
		sField.anz_oben=0;
		sField.anz_unten=0;
		sField.off_oben=0;
		sField.off_unten=0;
		

		if(abs(step)>5) {									//routine for strinking

			tField=getField(train[i]+ step/2);

			if(tField.anz_oben*k>0) {				//roof wrong color - swap cellar2roof
				tField.anz_unten=tField.anz_oben;
				tField.off_unten=tField.off_oben;
				tField.anz_oben=0;
				tField.off_oben=0;
			}



			if((fField.off_unten>>abs(fField.anz_unten))%2){
				tField.off_oben=tField.off_oben<<1;
				tField.off_oben++;
				fField.off_unten=fField.off_unten ^ 1<<abs(fField.anz_unten);
			}
			tField.anz_oben=tField.anz_oben-k;		//add one to the top
			fField.anz_unten=fField.anz_unten + k;	//delete one from the cellar
			
			game[tField.number]=tField;			
		}
		
		game[fField.number]=fField;
		game[sField.number]=sField;
	
	}
	
	return train;
}

bool board::in_temp(int field) {
	if(tMove.empty()) {
		return false;
	}
	for(int i=0;i<(int)tMove.size();i++) {
		if(tMove[i].number==field)
			return true;
	}

	return false;
}

field_t board::getEField(int nr) {

	if(nr<0 || nr>24) {
		debug("Ups, getEField got a strange field: "<< nr);
	}

	if(!tMove.empty()) {
		//do we have a changed field?
		fieldvec_it_t it=tMove.end()-1;
		for(;it>=tMove.begin();it--) {
			if(it->number == nr) {
				return *it;
			}
		}
	}
	//now can can savely return the original field
	return game[nr];

}
