// Copyright (C) 2006 The Ascal Authors - see AUTHORS for more information
//  
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software 
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#include "gozoai.hpp"
#include <iostream>
#include "helper.h"
#include <cstdlib>
#include <time.h>
#include <sstream>
#include "math.h"

#define GOZO_FACTOR 0.8
#define MY_MOBILITY 0.9

#define SHORT_MOVE_POSSIBLE 2
#define LONG_MOVE_POSSIBLE 8
#define OFF_SHORT_MOVE_POSSIBLE 4
#define OFF_LONG_MOVE_POSSIBLE 10

#define ME_ROOF 5
#define ME_OFFICER_ROOF 4
#define ME_OFFICER_ROOF_TOP 6

#define ME_CELLAR -2
#define ME_OFFICER_CELLAR -1
#define ME_OFFICER_CELLAR_TOP 1.1


#define OTHER_ROOF -5
#define OTHER_OFFICER_ROOF -2
#define OTHER_OFFICER_ROOF_TOP 6

#define OTHER_CELLAR 3
#define OTHER_OFFICER_CELLAR 3
#define OTHER_OFFICER_CELLAR_TOP 2

#define VICTORY_BONUS 1000 //added to the summed_mana

#define DEPTH 6 //should be an even number

void gozoai::print_move(ourmove_t * move)
{
	std::vector < int >::iterator it_move;
	std::ostringstream temp_stream;
	for (it_move = move->begin(); it_move != move->end(); it_move++) {
	temp_stream << *it_move << " ";
	}
	debug(temp_stream.str());

}


void gozoai::build_library1(std::vector < ourmove_t > *library,
				ourmove_t * move, int size, int last)
{
	if (library != NULL) {
	if (size > 0) {
		if (last == -1)
		for (int i = 0; i <= 24; i++) {
			move->push_back(i);
			build_library1(library, move, size - 1, i);
			move->pop_back();
		} else {
		for (int i = 0; i <= 24; i++) {
			if (abs(last - i) == 3 || abs(last - i) == 4
			|| abs(last - i) == 6 || abs(last - i) == 8) {
			move->push_back(i);
			build_library1(library, move, size - 1, i);
			move->pop_back();
			}
		}
		}
	} else {
		library->push_back(*move);
	}
	}
}

void gozoai::build_library(std::vector < ourmove_t > *library,
			   ourmove_t * move, int size, int last)
{
	if (library != NULL) {
	if (size > 0) {
		if (last == -1)
		for (int i = 0; i <= 24; i++) {
			move->push_back(i);
			build_library(library, move, size - 1, i);
			move->pop_back();
		} else {
		for (int i = 0; i <= 24; i++) {
			if (abs(last - i) == 6 || abs(last - i) == 8) {
			move->push_back(i);
			build_library(library, move, size - 1, i);
			move->pop_back();
			}
		}
		}
	} else {
		library->push_back(*move);
	}
	}
}

gozoai::~gozoai() {
	if(moveTree)
		delete moveTree;
}

gozoai::gozoai(const bool initColor, board* newBoard)
:ourplayer(initColor)
{
	if(initColor)
		myColor=1;
	else
		myColor=-1;
	
	oColor=myColor;
	myBoard=newBoard;
	
	ourmove_t move;
	fallback_a = fallback_b = false;

	fallback_a = 0;
	fallback_b = 0;

	// first we create 'all' possible moves

	build_library1(&backup, &move, 2);
	debug("size of move library 'backup':");
	debug(backup.size());
	//int i=0;
	build_library(&backup_fallback_a, &move, 3, -1);


	// we never know!
	build_library(&backup_fallback_b, &move, 4);



	// this need (too) MUCH time!
	build_library(&backup_fallback_c, &move, 5);
	build_library(&backup_fallback_c, &move, 6);
	build_library(&backup_fallback_c, &move, 7);

	debug("size of move library 'backup_fallback_a':");
	debug(backup_fallback_a.size());
	debug("size of move library 'backup_fallback_b':");
	debug(backup_fallback_b.size());
	debug("size of move library 'backup_fallback_c':");
	debug(backup_fallback_c.size());
	memory = backup;

	// before we use rand() ...
	srand((int) time(NULL));
	
	newMoves=moveTree=0;
}

ourmove_t gozoai::getourmove(move_signal sig)
{
	ourmove_t move;


	if(sig != NORMAL) {
		debug("Falling back to random mode");
		//fallback - if the last move was shitty than we use our random function
		//cool, he? ;)
		if (first_try_flag) {
			// previous move was allowed so ...
			// many moves to try
			memory = backup;
			first_try_flag=false;
	
			// first we use our standard move database
			fallback_a = 0;
			fallback_b = 0;
		} else {
		//previous ai move not allowed!

		  if(!fallback_a && memory.size() == 0){
		    debug("switching to fallback_a library");
		    memory = backup_fallback_a;
		    fallback_a = 1;
		  }
		  if(fallback_a && (memory.size() == 0) && !fallback_b){
		    debug("switching to fallback_b library");
		    memory = backup_fallback_b;
		    fallback_b = 1;
		  }    
		  if(fallback_b && memory.size() == 0){
		    debug("switching to fallback_c library");
		    memory = backup_fallback_c;
		  }
		}

		int r = rand() % memory.size();
		move = memory[r];
		memory.erase(memory.begin() + r);

		return move;
	}

	first_try_flag=true;

	//remove the old cruft
	if(moveTree)
		delete moveTree;
	newMoves = new std::list<gozoai_sit>;
		
	//get a recent board
	newGame=myBoard->getBoard();
	
	//build the first situation
	find_ways(newGame);
	//go the single possible way - don't let the user wait
	if(newGame.size()==1) {
		debug("only-one-move-possible-shortcut");
		return newMoves->begin()->train;
	}
	
	moveTree=newMoves;

	debug("after the first moves");
	//find all possibilities
	for(int i=1;i<DEPTH;i++) {
		newMoves=moveTree;
		oColor=(i%2) ? myColor: myColor*(-1);
		debug("digging level " << i);
		dig_deeper(i);
	}
	newMoves=moveTree;
	debug("digged deep enough - now the evaluation");
	collect_mana_from_the_ground(DEPTH-1);
	
	std::list<gozoai_sit>::iterator newMovesIt=moveTree->begin();
	std::list<gozoai_sit>::iterator fattest_mana=moveTree->begin();
	for(;newMovesIt!=moveTree->end();++newMovesIt) {
		debug("We look on a move starting at "<< newMovesIt->train[0] <<" with "<<newMovesIt->mana<<" of Mana");
		if(newMovesIt->victory){
			debug("choosing move leading to victory!");
			fattest_mana=newMovesIt;
			break;
		}
		if( newMovesIt->summed_mana > fattest_mana->summed_mana  ) {
			fattest_mana=newMovesIt;
		}
	}	
	//FIXME here we need some sugar: perhaps choose between the best 2 ones
	//or make a list of the last, say 2 moves und if we try to do it again - choose another one
	debug("and the winner is move from "<< fattest_mana->train[0]);

	return fattest_mana->train;

}

void gozoai::dig_deeper(int depth) {
	debug("dig deeper with a depth of "<<depth);
	if(depth==0) {
		debug("lets look for new moves");
		find_ways(newGame);
		return;
	}
	
	std::list<gozoai_sit>::iterator nMovesIt=newMoves->begin();
	
	//we have to save the end 'cause newMoves is changed in every step
	std::list<gozoai_sit>::iterator nMovesEnd=newMoves->end();
	--depth;
	for(;nMovesIt!=nMovesEnd;++nMovesIt) {
		if(!(nMovesIt->nextMoves)) {
			//debug("creating a new list in the tree");
			nMovesIt->nextMoves = new std::list<gozoai_sit>;
		}
		newMoves=(nMovesIt->nextMoves);
		newGame=nMovesIt->board;
		dig_deeper(depth);
		//debug("diving up");
		
		//now much new mana should be made and we can sum it up
		/*(nMovesIt->summed_mana)=0;
		getManaIt=nMovesIt->nextMoves->begin();
		for(;getManaIt != nMovesIt->nextMoves->end();++getManaIt) {
			nMovesIt->summed_mana+=getManaIt->summed_mana;
		}

		//now we construct the new mana consisting of 50% current mana und 50% of the deeper ones
		//FIXME: that is brain damage - we schould only consider our BEST possible move
		int size=nMovesIt->nextMoves->size();
		if(size!=0)
			nMovesIt->summed_mana=0.5*nMovesIt->mana+0.5/size*nMovesIt->summed_mana;
		else
			nMovesIt->summed_mana=nMovesIt->mana;*/
		//FIXMEvic: here should go the victories as well
		//debug("after deep digging");
	}
	
}

void gozoai::find_ways(fieldvec_t newGame) {
	
	forced_flag=false;
	
	//let's begin with a zero move
	ourmove_t voidMove;

	for(int i=0;i<25;i++) {
		if(newGame[i].anz_oben*oColor>0) {
		  how_do_you_like(i,newGame,voidMove);
		}
	}
	
	//FIXME: it should be possible to save the position where we take notice
	// of a long move and only go through the first ones up to that
	
	//kick out the short moves if there was a long one
	std::list<gozoai_sit>::iterator newMovesIt=newMoves->begin(),tempIt;
	if(forced_flag) {
		for(;newMovesIt!=newMoves->end();) {
			if(!(newMovesIt->longer)){
				tempIt=newMovesIt; //otherwise we had to reinit the iterator
				++newMovesIt;
				newMoves->erase(tempIt);
				continue;
			}
			++newMovesIt;
		}
	}
}

gdouble gozoai::collect_mana_from_the_ground(int depth) {
	debug("collecting mana from "<< depth);
	std::list<gozoai_sit>::iterator nMovesIt=newMoves->begin();
	
	if(depth==0) {
		debug("realized the null depth");
		std::list<gozoai_sit>::iterator newMovesIt=newMoves->begin();
		std::list<gozoai_sit>::iterator fattest_mana=newMoves->begin();
		for(;newMovesIt!=newMoves->end();++newMovesIt) {
			if(newMovesIt->victory){
				debug("choosing move leading to victory!");
				fattest_mana=newMovesIt;
				fattest_mana->summed_mana+=VICTORY_BONUS;
				break;
			}
			if( newMovesIt->summed_mana > fattest_mana->summed_mana  ) {
				fattest_mana=newMovesIt;
			}
		}
		debug("our hottest move is "<< fattest_mana->summed_mana);
		return fattest_mana->summed_mana;
	}
	
	//we have to save the end 'cause newMoves is changed in every step
	std::list<gozoai_sit>::iterator nMovesEnd=newMoves->end();
	--depth;
	
	//dive deep and remember the mana
	for(;nMovesIt!=nMovesEnd;++nMovesIt) {
		newMoves=(nMovesIt->nextMoves);
		if(depth%2==1)//if the next iteration is a move from the enemy
			nMovesIt->summed_mana=collect_mana_from_the_ground(depth);
		else
			nMovesIt->summed_mana=0.4*nMovesIt->mana+0.6*collect_mana_from_the_ground(depth);
	}
	
	//now let's find the best one 
	std::list<gozoai_sit>::iterator newMovesIt=newMoves->begin();
	std::list<gozoai_sit>::iterator fattest_mana=newMoves->begin();
	for(;newMovesIt!=newMoves->end();++newMovesIt) {
		if(newMovesIt->victory){
			debug("choosing move leading to victory!");
			fattest_mana=newMovesIt;
			fattest_mana->summed_mana+=VICTORY_BONUS;
			break;
		}
		if( newMovesIt->summed_mana > fattest_mana->summed_mana  ) {
			fattest_mana=newMovesIt;
		}
	}
	return fattest_mana->summed_mana;
		
}

bool gozoai::how_do_you_like(int i,fieldvec_t nGame, ourmove_t lastMove) {
	//debug("Evaluating field " << i << " with " << game[i].anz_oben);
	//i is the field we have to wonder about
	//LENT_FROM board victory
	//bool forced_flag=false;
//	debug("we look at field" << i);
	fieldvec_t game=nGame;
	if(!lastMove.empty()) {
		move(lastMove,&game);
	}
//	debug(game[17].off_oben);
	
	guint foo=in_move(i,lastMove);
	bool able_to_move=false;

	int next_field=i+oColor*6;
	if(	!out_of_bounds(i, oColor*6)&& !game[next_field].anz_oben
		&& game[i+oColor*3].anz_oben*oColor<0 && !(foo&1<<0)) {

		//debug("Allowed to move 6");
		forced_flag=true;
		able_to_move=true;
		how_do_you_like_worker(i,nGame,next_field,lastMove);


	}
	next_field=i+oColor*8;
	//debug("foo vergleich: "<<  !out_of_bounds(i, oColor*8) << !game[next_field].anz_oben << game[i+oColor*4].anz_oben );
	if(	!out_of_bounds(i, oColor*8)&& !game[next_field].anz_oben
		&& game[i+oColor*4].anz_oben*oColor<0 && !(foo&1<<1)) {

		//debug("Allowed to move 8");
		forced_flag=true;
		able_to_move=true;
		how_do_you_like_worker(i,nGame,next_field,lastMove);

	}
	//debug(game[i].off_oben);
	if(game[i].off_oben % 2)
	{
		//debug("found officer");
		next_field=i-oColor*6;
		if(	!out_of_bounds(i, -(oColor*6))&& !game[next_field].anz_oben
			&& game[i-oColor*3].anz_oben*oColor<0 && !(foo&1<<2)) {

			forced_flag=true;
			able_to_move=true;
			//debug("Allowed to move -6");
			how_do_you_like_worker(i,nGame,next_field,lastMove);

		}

		next_field=i-oColor*8;
		if(	!out_of_bounds(i, -(oColor*8))&& !game[next_field].anz_oben
			&& game[i-oColor*4].anz_oben*oColor<0 && !(foo&1<<3)) {

			//debug("Allowed to move -8");
			able_to_move=true;
			forced_flag=true;
			how_do_you_like_worker(i,nGame,next_field,lastMove);
		}

	}

	// only check small moves if the first moves were not there
	// AND we don't have forced moves at all
	if(!forced_flag){
		next_field=i+oColor*3;
		if(!out_of_bounds(i, oColor*3)&& !game[next_field].anz_oben && !(foo&1<<4)) {
			//debug("Allowed to move 3");
			how_do_you_like_worker(i,nGame,next_field,lastMove);
		}
		next_field=i+oColor*4;
		if(!out_of_bounds(i, oColor*4)&& !game[next_field].anz_oben && !(foo&1<<5)) {
			//debug("Allowed to move 4");
			how_do_you_like_worker(i,nGame,next_field,lastMove);
		}

		if(game[i].off_oben % 2)
		{
			next_field=i-oColor*3;
			if(!out_of_bounds(i, -(oColor*3))&& !game[next_field].anz_oben && !(foo&1<<6)) {
				//debug("Allowed to move -3");
				how_do_you_like_worker(i,nGame,next_field,lastMove);
			}
			next_field=i-oColor*4;
			if(!out_of_bounds(i, -(oColor*4))&& !game[next_field].anz_oben && !(foo&1<<7)) {
				//debug("Allowed to move -4");
				how_do_you_like_worker(i,nGame,next_field,lastMove);
			}

		}
	}
	return able_to_move;
}

void gozoai::how_do_you_like_worker(int i,fieldvec_t game, int next_field,ourmove_t lastMove) {

	debug("How do we like "<< i);
	bool longer=abs(next_field-i)>4;
	fieldvec_t newGame=game;
	ourmove_t nMove=lastMove;

	if(!lastMove.empty())
		longer=true;
	else
		nMove.push_back(i);


	nMove.push_back(next_field);
	move(nMove,&newGame);
	
	

	
	bool flag=false;
	
	if(longer) {
		//debug("Long move - let's have a look for next moves");
		flag=how_do_you_like(next_field,game,nMove);
	}
	
	//debug("Koennen wir weiter schlagen: "<<flag);
	if(!flag) {
		//if we don't have to go on
		//now the move is over and it's possible to build officers
		build_officer(next_field,&newGame);
		
		//debug("Nein, also neuer possible move");
/*#ifdef DEBUG
		for(int j=0;j<(int)nMove.size();j++) {
			debug(nMove[j]);
		}
#endif*/

		//debug("Before producing the mana");

		gozoai_sit cool_new_move;
		bool victory=0;
		if(myColor*oColor>0){
		//only rate if we look on an own move!!
			gdouble mana=produce_mana(newGame, &victory);
		
			//debug("And this great move from " << nMove[0] <<" has "<< mana <<" of mana and is long: "<<longer);
			cool_new_move.train=nMove;
			cool_new_move.summed_mana=cool_new_move.mana=mana;
		}
		else {
			cool_new_move.summed_mana=cool_new_move.mana=0;
		}
		cool_new_move.longer=longer;
		cool_new_move.victory=victory;
		cool_new_move.summed_victory=(gdouble)cool_new_move.victory;
		cool_new_move.board=newGame;		
		newMoves->push_back(cool_new_move);
	}

}

guint gozoai::in_move(int field, ourmove_t move) {
	guint foo=0;
	int middle;
	for(int i=0;i<(int)move.size();i++) {
		middle=(move[i+1]-move[i])/2+move[i];
		if(middle==field+oColor*3)
			foo|=1<<0;
		else if(middle==field+oColor*4)
			foo|=1<<1;
/*		else if(move[i]==field+oColor*3)
			foo|=1<<2;
		else if(move[i]==field+oColor*4)
			foo|=1<<3;*/
		else if(middle==field-oColor*3)
			foo|=1<<4;
		else if(middle==field-oColor*4)
			foo|=1<<5;
/*		else if(move[i]==field-oColor*3)
			foo|=1<<6;
		else if(move[i]==field-oColor*4)
			foo|=1<<7;*/
	}
	return foo;
}

void gozoai::update(ourmove_t move)
{
	debug("update called - ignoring it since we get the real board from our friend the logic");
	//move(move);
}

void gozoai::move(ourmove_t train, fieldvec_t* game) {
//LENT_FROM board check_move and move

	//debug("Start of move - train_start: "<< train[0] << " that ist oben "<< (*game)[train[0]].anz_oben);


	int k;
	//crazy conversion 'cause of bad arrangement
	if(game->at(train[0]).anz_oben<0) k=-1;
	else k=1;

	int i=0,step;
	field_t tField={0},fField,sField;
	for(;i<(int)train.size()-1;i++){
		step=train[i+1]-train[i];
		
	
	
		sField=game->at(train[i]);
		fField=game->at(train[i+1]);
		
		fField.anz_oben=sField.anz_oben;
		fField.anz_unten=sField.anz_unten;
		fField.off_oben=sField.off_oben;
		fField.off_unten=sField.off_unten;
		
		sField.anz_oben=0;
		sField.anz_unten=0;
		sField.off_oben=0;
		sField.off_unten=0;
		//debug("changing fields " << train);
		
	
		if(abs(step)>5) {									//routine for strinking
	
			tField=game->at(train[i]+ step/2);
			tField.anz_oben=tField.anz_oben+k;		//delete one from the top
	
			fField.anz_unten=fField.anz_unten - k;	//add one to the cellar
			
			if(tField.off_oben%2){
				fField.off_unten = fField.off_unten | 1<<(abs(fField.anz_unten)-1);
			}
			
			tField.off_oben=tField.off_oben>>1;		//remove the old officer
	
			if(tField.anz_oben==0) {				//roof empty - swap cellar2roof
				tField.anz_oben=tField.anz_unten;
				tField.off_oben=tField.off_unten;
				tField.anz_unten=0;
				tField.off_unten=0;
			}
			(*game)[tField.number]=tField;
	
		}
		
		(*game)[sField.number]=sField;
		(*game)[fField.number]=fField;
}

}

void gozoai::build_officer(int field, fieldvec_t* game){
	int k=oColor;

	field_t tField;

	if( ( (field<25 && field>20 && k == 1) ||
		  (field>-1 && field<4 && k == -1) ) &&
	 	!(game->at(field).off_oben % 2) )
												//build officer
	{
			debug("Building Officer");
			tField=game->at(field);
			tField.off_oben++;
			(*game)[tField.number]=tField;
	}
}

gdouble gozoai::produce_mana(fieldvec_t game, bool* victory) {

	debug("Producing mana");
	gdouble weightMe=0, weight=0, mobility=0,mobilityMe=0, temp;

	number_players = 0;
	number_no_mobility = 0;
	*victory = 0;

	for(int i=0;i<25;i++) {
		temp=weigh(game[i]);
		if(temp>0)
			weightMe+=temp;
		else
			weight+=temp;
		temp=can_move(i,game);
		if(temp>0)
			mobilityMe+=temp;
		else
			mobility+=temp;
		//
		/*mana_mania+=weight;
		mobility*/
	}

	if(number_players == number_no_mobility) 
	  *victory = 1;
	weightMe*=mobilityMe*GOZO_FACTOR;
	weight*=mobility;
		
	debug("Game has a MeMana of "<< weightMe <<" and OtherMana "<<weight);
	return weightMe-weight;

}

gdouble gozoai::weigh(field_t field) {
	gdouble weight=0, weightc=0;
	if(!field.anz_oben){
		//debug("WEIGHING: this was the shortcut");
		return weight;
	}
	if(field.anz_oben*oColor>0) {
		weight+=abs(field.anz_oben)*ME_ROOF;
		
		weight+=bit2number(field.off_oben)*ME_OFFICER_ROOF;
		//officer is that important that he changes the whole tower!!!
		if((field.off_oben%2))
			weight*=ME_OFFICER_ROOF_TOP;
		
		weightc+=abs(field.anz_unten)*OTHER_CELLAR;
		weightc+=bit2number(field.off_unten)*OTHER_OFFICER_CELLAR;
		if(field.off_unten%2)
			weightc*=OTHER_OFFICER_CELLAR_TOP;
	}
	else {
		weight+=abs(field.anz_oben)*OTHER_ROOF;
		weight+=bit2number(field.off_oben)*OTHER_OFFICER_ROOF;
		if(field.off_oben%2)
			weight*=OTHER_OFFICER_ROOF_TOP;
		
		weightc+=abs(field.anz_unten)*ME_CELLAR;
		weightc+=bit2number(field.off_unten)*ME_OFFICER_CELLAR;
		if(field.off_unten%2)
			weightc*=ME_OFFICER_CELLAR_TOP;		
	}
	return weight+weightc;
}

gdouble gozoai::can_move(int i,fieldvec_t game) {

	gdouble mana=0;

	if(game[i].anz_oben==0)
		return mana;

	int k;
	gdouble factor; 
	if(game[i].anz_oben>0) {
		k=1;
	}
	else{
		k=-1;
	}

	if(k==myColor) //FIXME
		factor=MY_MOBILITY;
	else{
		number_players++;
		factor=-1;
	}

	//LENT_FROM board victory
	if(!out_of_bounds(i, k*3)&& !game[i+k*3].anz_oben) mana+=SHORT_MOVE_POSSIBLE;
	if(!out_of_bounds(i, k*4)&& !game[i+k*4].anz_oben) mana+=SHORT_MOVE_POSSIBLE;
	if(	!out_of_bounds(i, k*6)&& !game[i+k*6].anz_oben
		&& game[i+k*3].anz_oben*k<0)
			mana+=LONG_MOVE_POSSIBLE;
	if(	!out_of_bounds(i, k*8)&& !game[i+k*8].anz_oben
		&& game[i+k*4].anz_oben*k<0)
			mana+=LONG_MOVE_POSSIBLE;

	if(game[i].off_oben % 2)
	{
		if(!out_of_bounds(i, -(k*3))&& !game[i-k*3].anz_oben) mana+=OFF_SHORT_MOVE_POSSIBLE;
		if(!out_of_bounds(i, -(k*4))&& !game[i-k*4].anz_oben) mana+=OFF_SHORT_MOVE_POSSIBLE;
		if(	!out_of_bounds(i, -(k*6))&& !game[i-k*6].anz_oben
			&& game[i-k*3].anz_oben*k<0)
				mana+=OFF_LONG_MOVE_POSSIBLE;
		if(	!out_of_bounds(i, -(k*8))&& !game[i-k*8].anz_oben
			&& game[i-k*4].anz_oben*k<0)
				mana+=OFF_LONG_MOVE_POSSIBLE;
	}

	
	if(mana == 0) number_no_mobility++;


	return factor*mana;

}



void gozoai::update_all()
{

}
void gozoai::victory(bool player)
{

}

bool gozoai::out_of_bounds(int pos, int step) {
//LENT_FROM board out_of_bounds
	if(pos+step<0||pos+step>24) {
		return true;
	}

	if ((step == 8 || step == -6) && pos % 7 == 6) {		//diese 4 if's stellen sicher, dass der stein nicht über den feldrand gesetzt wird
		return true;
	}

	if ((step == -4 || step == 3 || step == 6 || step == -8) && pos % 7 == 0) {
		return true;
	}

	if ((step == -3 || step == 4 || step == -6 || step == 8) && pos % 7 == 3) {
		return true;
	}

	if ((step == 6 || step == -8) && pos % 7 == 4) {
		return true;
	}
	return false;
}


int gozoai::bit2number(int i) {
	int k=0;
	while(i) {
		if(i%2)
			k++;
		i=i>>1;	
	}
	return k;
}
