// Copyright (C) 2006 The Ascal Authors - see AUTHORS for more information
//  
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software 
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifndef _gozoai_HPP
#define _gozoai_HPP

#include "ourplayer.hpp"
#include "ourmove.hpp"
#include "ourobserver.hpp"
#include "shareddata.hpp"
#include "helper.h"
#include "field.hpp"
#include <list>
#include "gozoai_sit.hpp"


class gozoai : public ourplayer, public ourobserver {
public:
	/** The computer player needs a color when it's constructed.
		@param initColor The player's color can be 0 or 1
	*/
	gozoai(const bool initColor,board* newBoard);
	
	~gozoai();
	/** Creates a new move. 
		@param sig Indicates that the previous move was wrong
	*/
	virtual ourmove_t getourmove(move_signal sig = NORMAL);
  
	virtual void update (ourmove_t move);
	virtual void update_all ();
	virtual void victory(bool player);
  
private:
	void build_library(std::vector<ourmove_t>* library, ourmove_t* move, int size, int last=-1);
	void build_library1(std::vector<ourmove_t>* library, ourmove_t* move, int size, int last=-1);

	void print_move(ourmove_t* move);
	
	/** walks down the pathes and shouts for find_ways on every situation
		in the given depth
		@param depth The depth to look in
	*/
	void dig_deeper(int depth);
	
	gdouble collect_mana_from_the_ground(int depth);
	
	gdouble produce_mana(fieldvec_t game, bool* victory);
	gdouble weigh(field_t field);
	gdouble can_move(int field, fieldvec_t game);
	void move(ourmove_t train, fieldvec_t* game);
	
	bool how_do_you_like(int i,fieldvec_t game,ourmove_t lastMove);
	void how_do_you_like_worker(int i,fieldvec_t game, int next_field,ourmove_t lastMove);
	
	guint in_move(int field, ourmove_t move);
	void build_officer(int field, fieldvec_t* game);
	
	bool out_of_bounds(int pos, int step);
	int bit2number(int i);
	
	/** Evaluates the situation on the board newGame and saves it in newMoves
	*/
	void find_ways(fieldvec_t newGame);
	
	/*fallback random mode*/
	//===========================
	std::vector<ourmove_t> memory;
	std::vector<ourmove_t> backup;
	std::vector<ourmove_t> backup_fallback_a;
	std::vector<ourmove_t> backup_fallback_b;
	std::vector<ourmove_t> backup_fallback_c;
	bool first_try_flag;
	bool fallback_a;
	bool fallback_b;
	//===========================
	
	
	board* myBoard;
	
	/** -1 and we are white, +1 and we are black
	*	\pre ->much easier to handle than 0 and 1
	*/
	int myColor;
	
	/* the currently used color
	*/
	int oColor;
	
	//the new moves in 1 round
	std::list<gozoai_sit> *newMoves;
	
	//the board before the situation that will be compiled in newMoves
	fieldvec_t newGame;
	
	//and all the collected rounds
	std::list<gozoai_sit> *moveTree;
	
	//is there a forced move?
	bool forced_flag;

	//victory detection
	int number_players;
	int number_no_mobility;
};

#endif
