/*
** Copyright (C) 2006 The Ascal Authors - see AUTHORS for more information
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
** 
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software 
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include "gui.hpp"
#include "helper.h"
#include "shareddata.hpp"
#include "config.h"
#include <string>

gui::gui(shareddata *newData): GuiBoard() {
	//clean, cleaner - pointer ;)
	canvas_ptr=0;
	canvas_layer=0;
	data=newData; OurLogik=0;
}

void gui::init() throw (Glib::Error) {
	
	try {
		gladeXml = Gnome::Glade::Xml::create(ASCAL_PIXMAPS_DIR"/ascal.glade");
	}
	catch (const Gnome::Glade::XmlError& err) {
		std::cerr << err.what() << std::endl;
		on_asc_kill(1);		
	}
	
	gladeXml->get_widget("window1", myWindow);
	if(!myWindow) {
		//blabla
	}
	myWindow->set_icon_from_file(ASCAL_PIXMAPS_DIR"/pixmaps/ascal.png");
	myWindow->set_default_icon_from_file(ASCAL_PIXMAPS_DIR"/pixmaps/ascal.png");
	
	Gtk::HBox* hbox;
	gladeXml->get_widget("CanFrame",hbox);
	if(hbox) {
		Gnome::Canvas::init();
		canvas_ptr = new Gnome::Canvas::CanvasAA;
		if(!canvas_ptr) {
			die("Not enough memory");
			on_asc_kill(1);
		}
		hbox->add(*canvas_ptr);
		
		canvas_ptr->set_size_request(790,550);

		canvas_ptr->set_scroll_region(0,0,790,550);
		canvas_ptr->set_size_request(790,550);

		gladeXml->get_widget("Status",status);
		canvas_layer = new Gnome::Canvas::Group(*canvas_ptr->root());
		if(!status || !canvas_layer) {
			die("Not enough memory or the ascal.glade is broken - Check your installation");
			clean_up();
			throw Glib::Error();
		}
		//canvas_layer->move(3,17);
		canvas_layer->move(3,35);
		Gtk::ToolButton* QuitButton=0;
		gladeXml->get_widget("BackButton", QuitButton);
		QuitButton->set_sensitive(false);

		Gtk::ImageMenuItem* BackItem=0, *SaveAsMenu=0, *SaveMenu=0;
		gladeXml->get_widget("MenuBack", BackItem);
		BackItem->set_sensitive(false);
		
		gladeXml->get_widget("Save",SaveMenu);
		SaveMenu->signal_activate().connect(sigc::mem_fun(*this, &gui::save_click));
		SaveMenu->set_sensitive(false);
		gladeXml->get_widget("SaveAs",SaveAsMenu);
		SaveAsMenu->signal_activate().connect(sigc::mem_fun(*this, &gui::save_as_click));
		SaveAsMenu->set_sensitive(false);	
		
		
					
		GuiBoard.signal_asc_kill().connect( sigc::mem_fun(*this, &gui::on_asc_kill));
		GuiBoard.init(canvas_layer,status, QuitButton, BackItem, SaveMenu, SaveAsMenu, data); //init the board and tell it its layers...

	
		//We have to say gtk that we want to listen to this event!!
		canvas_ptr->add_events(Gdk::BUTTON_PRESS_MASK);
		canvas_ptr->signal_button_press_event().connect(
											sigc::mem_fun(	GuiBoard,
															&gui_board::on_click)
														);



		gladeXml->get_widget("QuitButton", QuitButton);
		if(QuitButton) {
			QuitButton->signal_clicked().connect(sigc::mem_fun(*this, &gui::quit_click) );
		}
		gladeXml->get_widget("NewButton", QuitButton);
		if(QuitButton) {
			QuitButton->signal_clicked().connect(sigc::mem_fun(*this, &gui::new_click) );
		}
		gladeXml->get_widget("BackButton", QuitButton);		
		if(QuitButton) {
			QuitButton->signal_clicked().connect(sigc::mem_fun(GuiBoard, &gui_board::back_click) );
		}
		
		Gtk::MenuItem* MenuInfo=0;
		gladeXml->get_widget("MenuInfo",MenuInfo);
		if(MenuInfo)
			MenuInfo->signal_activate().connect(sigc::mem_fun(*this,&gui::open_info));
		gladeXml->get_widget("MenuQuit",MenuInfo);
		if(MenuInfo)
			MenuInfo->signal_activate().connect(sigc::mem_fun(*this,&gui::quit_click));
		gladeXml->get_widget("MenuNew",MenuInfo);
		if(MenuInfo)
			MenuInfo->signal_activate().connect(sigc::mem_fun(*this,&gui::new_click));
		gladeXml->get_widget("MenuBack",MenuInfo);
		if(MenuInfo)
			MenuInfo->signal_activate().connect(sigc::mem_fun(GuiBoard, &gui_board::back_click));
		gladeXml->get_widget("MenuPrefs",MenuInfo);
		if(MenuInfo)
			MenuInfo->signal_activate().connect(sigc::mem_fun(*this, &gui::open_prefs));

		gladeXml->get_widget("Open",MenuInfo);
		if(MenuInfo)
			MenuInfo->signal_activate().connect(sigc::mem_fun(*this, &gui::open_click));

		gladeXml->get_widget("entry1",Entry1);
		gladeXml->get_widget("entry2",Entry2);
		gladeXml->get_widget("radiobutton2",RadButton);
		gladeXml->get_widget("PrefsClose",PrefQuitButton);
		
		gladeXml->get_widget("Prefs",PrefWindow);
		if(Entry1 && Entry2 && RadButton && QuitButton && PrefWindow) {
			Entry1->set_text(_("Player 1"));
			Entry2->set_text(_("Player 2"));
			RadButton->set_active(!data->getnumberai());
		
			Entry1->signal_changed().connect(sigc::mem_fun(*this, &gui::on_prefs_text1_change));
			Entry2->signal_changed().connect(sigc::mem_fun(*this, &gui::on_prefs_text2_change));
			PrefQuitButton->signal_clicked().connect(sigc::mem_fun(*this, &gui::on_prefs_close_click));
			RadButton->signal_toggled().connect(sigc::mem_fun(*this, &gui::on_prefs_radio_change));
			PrefWindow->signal_delete_event ().connect(sigc::mem_fun(*this, &gui::on_prefs_close));
		}
		
		myWindow->signal_delete_event ().connect(sigc::mem_fun(*this, &gui::on_delete));
		/*wichtig: das folgende zeigt dann auch die canvas sachen an!!*/
		myWindow->show_all();

		//sigc::slot<bool> my_slot = sigc::bind(sigc::mem_fun(GuiBoard, &gui_board::check_for_news));
		//sigc::connection conn =
		
		if(data->getnumberai() == 2){
		  Glib::signal_timeout().connect(sigc::mem_fun(GuiBoard, &gui_board::check_for_news),350);		
		}
		else{
		  Glib::signal_timeout().connect(sigc::mem_fun(GuiBoard, &gui_board::check_for_news),5);
		}

	}
	else {
		die("The ascal.glade is broken - Check your installation");
		clean_up();
		throw Glib::Error();
	}
	OurLogik = new logik(data);
	if(!OurLogik) {
		die("Not enough memory to create a new game");
		clean_up();
		throw Glib::Error();
	}	

	thrd_logic=Glib::Thread::create(sigc::mem_fun(OurLogik, &logik::run), 0, true, true, Glib::THREAD_PRIORITY_LOW);

}

gui::~gui() {
	clean_up();
}

void gui::clean_up() {
	
	debug("Cleaning Up");
	if(canvas_ptr)
		delete canvas_ptr;
	if(canvas_layer)
		delete canvas_layer;
	if(OurLogik)
		delete OurLogik;

	/*MoH: FIXME: more to clean?! */
	debug("Everything is clean");
}

void gui::on_asc_kill(int early) throw (Glib::Error) {
	debug("asc_kill called!");
	if(early) {
		clean_up();
		throw Glib::Error();
	}
	quit();
}

void gui::quit_click() {
	quit();
}

bool gui::on_delete(GdkEventAny* a) {
	quit();
	return true;
}

void gui::quit() {
	//wait till the logic kills itself
	data->kill_logic();
	thrd_logic->join();
	if(myWindow) {
		myWindow->hide();
	}
}

void gui::open_info() {
#ifndef GTKMM_24
	m_Dialog.set_transient_for(*myWindow);
	m_Dialog.set_name(_("Ascal"));
	m_Dialog.set_version(PACKAGE_VERSION);
	m_Dialog.set_copyright("(c) 2006 The Ascal Authors");
	m_Dialog.set_comments(_("This is a simple Gnome Version of Laska."));
	m_Dialog.set_logo(Gdk::Pixbuf::create_from_file(ASCAL_PIXMAPS_DIR"/pixmaps/ascal_large.png"));

	std::list<Glib::ustring> list_authors;
	list_authors.push_back("Nils Kanning");
	list_authors.push_back("Patrick Haase");
	list_authors.push_back("Steffen Klemer");
	list_authors.push_back("Thorsten Weikert");
	m_Dialog.set_authors(list_authors);
	m_Dialog.show();
	m_Dialog.present();
#else
	status->push("Not implemented for your old gtk version yet.");
	debug("Old GTK version version with a status");
#endif
}

void gui::new_click() {
	data->kill_logic();
	GuiBoard.changePlayerName(1, Entry2->get_text());
	GuiBoard.changePlayerName(0, Entry1->get_text());
	status->pop();
	thrd_logic->join();

	data->clean_up();
	GuiBoard.clear_board();

	thrd_logic=Glib::Thread::create(sigc::mem_fun(OurLogik, &logik::run), 0, true, true, Glib::THREAD_PRIORITY_LOW);
}

void gui::save_click() {
	if(last_filename.size()==0) {
		save_as_click();
	}
	else {
		data->save_history(last_filename);
		status->push(_("Game Saved"));
	}

}

void gui::save_as_click() {

  
	Gtk::FileChooserDialog dialog(_("Please choose a file"), Gtk::FILE_CHOOSER_ACTION_SAVE);
	dialog.set_transient_for(*myWindow);

	//Add response buttons the the dialog:
	dialog.add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);
	dialog.add_button(Gtk::Stock::SAVE, Gtk::RESPONSE_OK);
	dialog.set_default_response(Gtk::RESPONSE_OK);

	//Add filters, so that only certain file types can be selected:

	Gtk::FileFilter filter_text;
	filter_text.set_name(_("Ascal Games"));
	filter_text.add_pattern("*.asc");
	dialog.add_filter(filter_text);

	Gtk::FileFilter filter_any;
	filter_any.set_name(_("Any files"));
	filter_any.add_pattern("*");
	dialog.add_filter(filter_any);
	
	dialog.set_do_overwrite_confirmation();
	if(!last_filename.empty()){
		dialog.set_filename(last_filename);
	}
	//Show the dialog and wait for a user response:
	int result = dialog.run();

	//Handle the response:
	switch(result)
	{
	case(Gtk::RESPONSE_OK):
	{
	  last_filename = dialog.get_filename(); //Notice that it is a std::string, not a Glib::ustring.
	  debug(last_filename);
	  data->save_history(last_filename);
	  status->push(_("Game Saved"));
	  break;
	}
    case(Gtk::RESPONSE_CANCEL):
    {
      break;
    }
  }
 
}

void gui::open_click() {
	std::string filename;
  
  
	Gtk::FileChooserDialog dialog(_("Please choose a file"), Gtk::FILE_CHOOSER_ACTION_OPEN);
	dialog.set_transient_for(*myWindow);

	//Add response buttons the the dialog:
	dialog.add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);
	dialog.add_button(Gtk::Stock::OPEN, Gtk::RESPONSE_OK);
	dialog.set_default_response(Gtk::RESPONSE_OK);

	//Add filters, so that only certain file types can be selected:

	Gtk::FileFilter filter_text;
	filter_text.set_name(_("Ascal Games"));
	filter_text.add_pattern("*.asc");
	dialog.add_filter(filter_text);

	Gtk::FileFilter filter_any;
	filter_any.set_name(_("Any files"));
	filter_any.add_pattern("*");
	dialog.add_filter(filter_any);

	//Show the dialog and wait for a user response:
	int result = dialog.run();

	//Handle the response:
	switch(result)
	{
	case(Gtk::RESPONSE_OK):
	{
		last_filename = dialog.get_filename(); //Notice that it is a std::string, not a Glib::ustring.
		data->kill_logic();
		GuiBoard.changePlayerName(1, Entry2->get_text());
		GuiBoard.changePlayerName(0, Entry1->get_text());
		thrd_logic->join();


		data->clean_up();
		GuiBoard.clear_board();
		data->open_history(last_filename);

		data->setnumberai(-1);

		thrd_logic=Glib::Thread::create(sigc::mem_fun(OurLogik, &logik::run), 0, true, true, Glib::THREAD_PRIORITY_LOW);

		break;
	}
    case(Gtk::RESPONSE_CANCEL):
    {
      break;
    }
  }
 
}

void gui::open_prefs() {
	gladeXml->get_widget("Prefs",PrefWindow);
	if(PrefWindow) { 
		PrefWindow->show(); 
	 }
}

bool gui:: on_prefs_close(GdkEventAny* a) {
	PrefWindow->hide();
	return true;
}

void gui:: on_prefs_close_click() {
	PrefWindow->hide();
}

void gui::on_prefs_radio_change() {
  if(RadButton->property_active()){
    debug("radio_change");
    data->reconfigure_players(0);
  }
  else{
    debug("radio_change");
    data->reconfigure_players(1);
  }

}
void gui::on_prefs_text1_change(){
	GuiBoard.changePlayerName(0, Entry1->get_text());
}
void gui::on_prefs_text2_change(){
	GuiBoard.changePlayerName(1, Entry2->get_text());
}
