/*
** Copyright (C) 2006 The Ascal Authors - see AUTHORS for more information
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
** 
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software 
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#ifndef GUI_HPP
#define GUI_HPP

#include  <sigc++/sigc++.h>
#include <libglademm/xml.h>
#include <gtkmm.h>
#include <libgnomecanvasmm.h>
#include "gui_board.hpp"
#include "logik.hpp"
#include "config.h"
#include <string>



class gui {

public:
	gui();
  gui(shareddata *newData);
	/** Initializes the GUI, loads the glade file, inits the canvas
		and throws Glib::Error when anything goes wrong	
	*/
	void init() throw (Glib::Error);
	virtual  ~gui();
	//void on_logik_update(int x, int y);
	
	Gtk::Window* myWindow;
	
protected:
	/**callback for the info menu item
		Builds the infobox
	*/
	void open_info();
	/** Callback for the click on quit
	*/
	void quit_click();
	
	/** Callback for the click on "New Game"
	*/
	void new_click();

	/** Callback for the click on "Save As"
	*/
	void save_as_click();
	void save_click();

	/** Callback for the click on "Open"
	*/
	void open_click();

	/** Callback for asc_kill events from our GuiBoard
		cleans up and quits
		@param 1 for overriding gtk and throwing an exception
	*/
	void on_asc_kill(int early) throw (Glib::Error);	
	
	/** Callback to open the prefs-Window
	*/
	void open_prefs();
	
	bool on_delete(GdkEventAny* a);
	
	
	bool on_prefs_close(GdkEventAny* a=0);
	void on_prefs_close_click();
	void on_prefs_radio_change();
	void on_prefs_text1_change();
	void on_prefs_text2_change();

	Glib::RefPtr<Gnome::Glade::Xml> gladeXml;
#ifndef GTKMM_24	
	Gtk::AboutDialog m_Dialog;
#endif
	
	Gnome::Canvas::Group* canvas_layer;
	

	Gnome::Canvas::CanvasAA* canvas_ptr;

	gui_board GuiBoard;
	

private:
	/** Hides the window and tries to shutdown gtk
	*/
	void quit();
	
	/** free all acquired space*/
	void clean_up();
	
	logik* OurLogik;
	shareddata* data;
	Glib::Thread *thrd_logic;
	Gtk::Statusbar* status;

	Gtk::RadioButton* RadButton;
	Gtk::Entry* Entry1;
	Gtk::Entry* Entry2;
	Gtk::Button* PrefQuitButton;
	Gtk::Window *PrefWindow;
	std::string last_filename;

};


#endif //GUI_HPP


