// Copyright (C) 2006 The Ascal Authors - see AUTHORS for more information
//  
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software 
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.


#ifndef GUI_BOARD_HPP
#define GUI_BOARD_HPP

#include <gtkmm.h>
#include <libglademm.h>
#include <libgnomecanvasmm.h>
#include <sigc++/sigc++.h>
#include <vector>
#include "ourmove.hpp"
#include "field.hpp"
#include "shareddata.hpp"
#include "config.h"

#define ASCAL_PIXMAPS_DIR ASCAL_DATA_PREFIX

/** Not much to say here - just the coordinates of a point on the canvas
*/
typedef struct {
	double x;
	double y;
} ko_t;

const gdouble GUI_STONE_HEIGHT=10;
const guint COLOR_WHITE=0xe43118ff;
const guint COLOR_BLACK=0x3248bbff;
const guint COLOR_NEUTRAL=0x666666ff;

class gui_board {
public:
	gui_board();

	/** Does all the Paintin on the graphical board
	* @param A Pointer to a canvas group to draw on
	* @param A Pointer to a status bar
	* @param A Pointer to the Back Button
	* @param A pointer to the shared data
	*/
  void init(Gnome::Canvas::Group* new_canvas_layer, Gtk::Statusbar* new_status, Gtk::ToolButton* new_back_button, Gtk::ImageMenuItem* new_menu_back,Gtk::ImageMenuItem* new_save, Gtk::ImageMenuItem* new_save_as, shareddata* newData);	
	
	~gui_board();
	
	/**Reacts on the clicks into the drawing area
	*/	
	bool on_click( GdkEventButton* ev);
	
	/** The signal we send when we think anything went wrong
	*/
	typedef sigc::signal<void,int> type_signal_asc_kill;
	
	/** The signal as an object
	*/
	type_signal_asc_kill signal_asc_kill();
	
	/** 
	*/
	bool check_for_news();

	/** Callback for the "back" function
	*/
	void back_click();
	
	/** Clears the board - to be used after a click on new game
	*/
	void clear_board();
	
	/** Change the displayed Name
		@param false for Player 2 true for Player 1
	*/
	void changePlayerName(bool player, Glib::ustring newName);

protected:
	type_signal_asc_kill my_signal_asc_kill;
  
private:
	
	/** Draws a circle :)
	* @param Color of the Circle
	* @param Number of the field
	* @param Size (0 is field-size, 1 a normal player, 2 a smaller one...)	
	*/
	//void draw_circle(PlayerColor_t color, int field, int depth);
	
	/** Returns the center of the given field
	*/
	ko_t find_circle(int field) const;
	
	/** This gives us the field at x:y or -1
	*/
	int wheres_the_pointer(gdouble x, gdouble y) const;
	
	void on_update(fieldvec_t changedFields);
	void on_request_move(request_t re);
	void on_request_move(move_signal sig);
	void add_to_field(int Stone, int Field);
	void clear_field(int Field);
	void upgrade(int Stone);
	void downgrade(int Stone);
	void victory(bool Player);
	bool fade();
	
	Glib::RefPtr<Gdk::Pixbuf> StoneBlack;
	Glib::RefPtr<Gdk::Pixbuf> StoneBlackHl;
	Glib::RefPtr<Gdk::Pixbuf> StoneWhite;
	Glib::RefPtr<Gdk::Pixbuf> StoneWhiteHl;
	Glib::RefPtr<Gdk::Pixbuf> HighlField;
	
	typedef struct {
		Gnome::Canvas::Group* canvas;
		int nrStones;
		bool Stones[22];
	} canvas_field_t;
	
	canvas_field_t fields[25];
	
	typedef struct {
		Gnome::Canvas::Pixbuf* pixb;
		bool Officer;
		int pos;
	} canvas_stones_t;
	
	canvas_stones_t stones[22]; 
	Gnome::Canvas::Pixbuf* hlcircles[25];
	std::list<int> hlcircles_alpha;

	Gtk::Statusbar* status;
	Gtk::ToolButton* back_button;
	Gtk::ImageMenuItem* menu_back;
	Gtk::ImageMenuItem* save;
	Gtk::ImageMenuItem* save_as;
	Gnome::Canvas::Pixbuf* bg_pb;
	Gnome::Canvas::Pixbuf* bg_bg_pb;
	Gnome::Canvas::Pixbuf* bg_yabg_pb;
	Glib::RefPtr<Gdk::Pixbuf> bg;
	Glib::RefPtr<Gdk::Pixbuf> bg_bg;
	Glib::RefPtr<Gdk::Pixbuf> yabg;
        Gnome::Canvas::Group* bg_layer;
	Gnome::Canvas::Group* board_layer;
	Gnome::Canvas::Group* boardbg_layer;
	Gnome::Canvas::Group* stone_layer;
	Gnome::Canvas::Group* highl_layer;
	
	Gnome::Canvas::Text *Player1Text;
	Gnome::Canvas::Text *Player2Text;
	
	
	
	int WaitForPlayer;
	
	shareddata *data;
	request_t Re;

	/** A new Move will be constructed in this one
	*/
	ourmove_t genMove;
};


#endif //GUI_BOARD_HPP
