/*
** Copyright (C) 2006 The Ascal Authors - see AUTHORS for more information
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
** 
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software 
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include "logik.hpp"
#include "board.hpp"
#include "helper.h"
#include "guiobserver.hpp"
#include "gozoai.hpp"

//#include "TournamentPlayer.hpp"


#include <vector>
#include <iostream>

logik::logik(shareddata * data):data(data)
{
    guiobserver* newobs=new guiobserver (data, &game);
    obs.push_back(newobs);
	players[0] = players[1] = 0;

	//init the move 2 ourmove arrays

	debug("The init of o2m");
	int j,i,k;

	for(i=0;i<25;i++) {
		j=i;
		k=0;
		while((!(k%2) && j>3) || ((k%2) && j>2)) {
			j-=(4-k%2);
			++k;
		}

		o2m[i].row=k;
		o2m[i].col=j*2+k%2;
		//debug(i << " = "<< o2m[i].row << " "<< o2m[i].col);
	}

	debug("Now the init of m20");
	
	//init the ourmove 2 move array
	for(j=0;j<7;j++)
		for(i=0;i<7;i++) {
			if((i%2 && !(j%2)) || (!(i%2) && j%2) )
				m2o[j][i]=0;
			else {
				m2o[j][i]= ((j+1)/2)*4 + (j-(j+1)/2)*3 + i/2;
				//debug(j << " " << i << " = "<< m2o[j][i]);
			}
		}


}

logik::~logik() {
	debug("make clean");
	if(players[1])
		delete players[1];
	if(players[0])
		delete players[0];
}

void logik::run()
{
  init_players();

    game.clear();

    debug("logic: logic started");

    unsigned int i=0,round=1;
    bool back_flag=false;
    move_signal last_sig=NORMAL;
    ourmove_t m;
    
    while (true) {
		debug("logic: new round");
		if(last_sig==BACK && data->getnumberai()==1 && !data->history.empty()) {
				m=game.goback(data->history.back());
				data->history.pop_back();
				last_sig=BACK2;
		}
		else {
			last_sig=NORMAL;
			do {
				//debug("Move holen von Player "<< round);
				m=players[round]->getourmove(last_sig);
				
				switch(m[0]) {
					case -1:
						debug("Got Kill signal");
						delete players[1];
						delete players[0];
						players[0] = players[1] = 0;
						return;
					case -2:
						debug("Got Back signal");
					if (!data->history.empty()) {
							m=game.goback(data->history.back());
							data->history.pop_back();
							data->reset_player_request();
							last_sig=BACK;
					}
					break;
					case -3:
						debug("Got: give_me_the_whole_board_signal");
						last_sig=NORMAL;
						for (i = 0; i < obs.size(); i++) {
							obs.front()->update_all();
						}
						m.clear();
						++round%=2;
						break;
					case -8: //you'll ask: why "8"? - braindamage at 3am ;)
					debug("got signal: resetting ai players!");
					back_flag=true;
						init_players();
						last_sig=RESET;
						data->reset_player_request();
						m.clear();
						++round%=2;
						break;
					default:
						last_sig=game.move(m, round);
				}
	
			} while(last_sig!=NORMAL && last_sig!=NORMAL_OFFICER && last_sig!=BACK && last_sig != RESET);
		}
		if(last_sig==RESET) {
			++round%=2;
			continue;
		}
		
		//remember all the moves
		if(last_sig!=BACK && last_sig!=BACK2) {
			if(last_sig==NORMAL_OFFICER) {
				m.push_back(-1);
				for(int i=0;i<(int)m.size();i++) {
					debug(m[i]);
				}
				data->history.push_back(m);
				m.pop_back();
				last_sig=NORMAL;
			}
			else
				data->history.push_back(m);
		}

		for (i = 0; i < obs.size(); i++) {
			debug("Updating observer " << i);
		    obs[i]->update(m);
		}
		
		//lets look whether the other player still has a move to do
		//if not inform the observers
		if(game.victory(!round))	{
		   for(i=0; i<obs.size(); i++ ){
			   obs[i]->victory(round);
		   }

			#ifndef DEBUG
		   data->history.clear();	//in the meantime we can only save the history after a victory if debug is enabled
		   #endif

		   return; //FIXMEmoh: in the meantime a back after a victory is perhaps possible!
		}
		
		++round%=2;

    }
}


void logik::init_players(){
	if(players[0])
		delete players[0];
	if(players[1])
		delete players[1];
	for(int l=obs.size();l>1;l--) {
		//remove all refs to observer ais
		obs.pop_back();
	}
    players[0] = players[1] = 0;
    //create tmep as a pointer to gozoai to get the possibility of representing it as
    //a player or an observer
    gozoai* tmep=0;

	switch(data->getnumberai()){
		case 1:
			players[1] = new human(WHITE, data);
			tmep=new gozoai(BLACK,&game);
			players[0] = (ourplayer*)tmep;
			obs.push_back((ourobserver*)tmep);
			break;
		case 2:
			tmep=new gozoai(WHITE,&game);
			players[1] = (ourplayer*)tmep;
			obs.push_back((ourobserver*)tmep);
			players[0] = new randomai(BLACK);
			break;
		default:
			players[1] = new human(WHITE, data);
			players[0] = new human(BLACK, data);
      }

	if (!players[0] || !players[1]) {
		die("Not enough memory to create my Players - will die");
		//FIXMEmoh - error handling here!
	}

    return;
}

Move logik::ourmove2move(ourmove_t m)  {
	Move n;

	for(int i=0;i<(int)m.size();i++) {
		n.push_back(o2m[m[i]]);
	}

	return n;
}

ourmove_t logik::move2ourmove(Move m) {
	ourmove_t n;
	Position pos;
	
	while(!m.empty()) {
		pos=m.front();
		m.pop_front();
		n.push_back(m2o[pos.row][pos.col]);
	}

	return n;

}
