// Copyright (C) 2006 The Ascal Authors - see AUTHORS for more information
//  
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software 
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifndef _MOVE_HPP
#define _MOVE_HPP

#include <list>

/** A Position on the Lasca Board. 
 * When starting the game, the basic layout of the board is as follows:
 *
 *      0   1   2   3   4   5   6  (col)
 *    -----------------------------
 *  0 | w |   | w |   | w |   | w |
 *    -----------------------------
 *  1 |   | w |   | w |   | w |   |
 *    -----------------------------
 *  2 | w |   | w |   | w |   | w |
 *    -----------------------------
 *  3 |   |   |   |   |   |   |   |
 *    -----------------------------
 *  4 | b |   | b |   | b |   | b |
 *    -----------------------------
 *  5 |   | b |   | b |   | b |   |
 *    -----------------------------
 *  6 | b |   | b |   | b |   | b |
 *    -----------------------------
 * (row)
 */
struct Position {
  int row, col;
};



/** 
 * A Lasca move is represented as an STL-list. 
 * To perform the move, the list is followed step by step.
 *
 * \par First list element
 * The first element of the list contains the starting point of the move. 
 * The tower found at that position moves to the second position in the list,
 * then to the third, and so on.
 * 
 * \par Last list element
 * The last element contains the position that is ultimately reached 
 * by the move
 */
typedef std::list<Position> Move;

#endif
