// Copyright (C) 2006 The Ascal Authors - see AUTHORS for more information
//  
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software 
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifndef _OBSERVER_H
#define _OBSERVER_H

#include "move.hpp"
#include <string>

using std::string;
/** 
 * An Observer of the game.
 * Observers are all entities that want to keep track of the moves
 * (e.g. output classes or computer player).
 * In particular, the graphical user interfaces must be derived from the 
 * abstract class observer, so that the tournament observer can access them.
 */
class Observer {
public:
  /** The constructor of an observer can be used to name the players.
   * \param namePlayer1 Name of the first player, playing the white tokens
   * \param namePlayer2 Name of the second player, playing the black tokens
   */
  Observer (string namePlayer1 = "Player 1", string namePlayer2 = "Player 2") 
    : player1 (namePlayer1), player2 (namePlayer2) {}

  /** virtual destructor needed because of abstract class */
  virtual ~Observer () {/* nothing to be done */} 

  /** Notify the observer of a move. 
   * The observer has to be notified of every move of either player.
   *
   * \param myMove The last move in the game
   */
  virtual void update (Move myMove) =0;
protected:
  string player1; /// Name of the first player, playing the white tokens
  string player2; /// Name of the second player, playing the black tokens
};

#endif
