// Copyright (C) 2006 The Ascal Authors - see AUTHORS for more information
//  
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software 
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifndef _PLAYER_HPP
#define _PLAYER_HPP

#include "move.hpp"

/** Global variable for the color white */
const bool WHITE = true;
/** Global variable for the color black */
const bool BLACK = false;


/** This interface must be implemented by every entity that will function as
 * a player, e.g. computer player, human player.
 * This means that every player has to be derived from the Player Interface.
 *
 * The tournament player will also implement this interface.
 */
class Player {
public:
  /** When constructing a player, 
   * it must be notified of the color it is playing.
   *
   * \param initColor Color of the player's stones. Default is white.
   */
  Player (const bool initColor = WHITE) : color (initColor) {}
  /** virtual destructor needed because of abstract class */
  virtual ~Player() { /* nothing to be done */}
  /** Returns the move made by the player. 
   * \pre The move must be correct!
   * \return Move
   */
  virtual Move getMove() =0;

protected:
  bool color;
};

#endif
