// Copyright (C) 2006 The Ascal Authors - see AUTHORS for more information
//  
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software 
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#include "randomai.hpp"
#include <iostream>
#include "helper.h"
#include <cstdlib>
#include <time.h>
#include <sstream>
#include "math.h"


// this function is just for debugging
void randomai::print_move(ourmove_t* move){
    std::vector<int>::iterator it_move;
    std::ostringstream temp_stream;
    for( it_move = move->begin(); it_move != move->end(); it_move++ ) {
      temp_stream << *it_move << " ";
    } 
    debug(temp_stream.str());

}

// this function is just for debugging
void randomai::print_library(std::vector<ourmove_t>* library){
  std::vector<ourmove_t>::iterator it_lib;
  ourmove_t temp_move;
  debug("printing move library:");
  for( it_lib = library->begin(); it_lib != library->end(); it_lib++ ) {    
    temp_move = *it_lib;
    print_move(&temp_move);
  } 
}

void randomai::build_library1(std::vector<ourmove_t>* library, ourmove_t* move, int size, int last){
  if(library != NULL){
    if(size > 0){
    	if(last==-1)
	      for(int i = 0; i <= 24; i++){
			move->push_back(i);
			build_library1(library, move, size - 1,i);
			move->pop_back();
	      }
	     else {
	     	for(int i = 0; i <= 24; i++) {
	     		if(abs(last-i)==3 || abs(last-i)==4 || abs(last-i)==6 || abs(last-i)==8) {
	     			move->push_back(i);
					build_library1(library, move, size - 1,i);
					move->pop_back();
	     		}
	     	}
	     }
    }
    else {
		library->push_back(*move);
    } 
  }
}

void randomai::build_library(std::vector<ourmove_t>* library, ourmove_t* move, int size,int last){
  if(library != NULL){
    if(size > 0){
    	if(last==-1)
	      for(int i = 0; i <= 24; i++){
			move->push_back(i);
			build_library(library, move, size - 1,i);
			move->pop_back();
	      }
	     else {
	     	for(int i = 0; i <= 24; i++) {
	     		if(abs(last-i)==6 || abs(last-i)==8) {
	     			move->push_back(i);
					build_library(library, move, size - 1,i);
					move->pop_back();
	     		}
	     	}
	     }
    }
    else {
		library->push_back(*move);
    } 
  }
}


randomai::randomai(const bool initColor) 
    : ourplayer(initColor) {
  ourmove_t move;
  fallback_a=fallback_b=false;

  fallback_a = 0;
  fallback_b = 0;

  // first we create 'all' possible moves
  
  build_library1(&backup, &move, 2);
  debug("size of move library 'backup':");
  debug(backup.size());  
  //int i=0;
  build_library(&backup_fallback_a, &move, 3,-1);
  /*for(;i<(int)backup_fallback_a.size();i++) {
  	if( abs(backup_fallback_a[i][0]-backup_fallback_a[i][1])!=6 ||
  		abs(backup_fallback_a[i][0]-backup_fallback_a[i][1])!=8 ||
  		abs(backup_fallback_a[i][1]-backup_fallback_a[i][2])!=6 ||
  		abs(backup_fallback_a[i][1]-backup_fallback_a[i][2])!=8 
  	) {
  		backup_fallback_a.erase(backup_fallback_a.begin()+i);
  		debug("erased number "<<i);
  	}
  
  }*/

  // we never know!
  build_library(&backup_fallback_b, &move, 4);
  
  

  // this need (too) MUCH time! 
  build_library(&backup_fallback_c, &move, 5);
  build_library(&backup_fallback_c, &move, 6);
  build_library(&backup_fallback_c, &move, 7);

  debug("size of move library 'backup_fallback_a':");
  debug(backup_fallback_a.size()); 
  debug("size of move library 'backup_fallback_b':");
  debug(backup_fallback_b.size()); 
  debug("size of move library 'backup_fallback_c':");
  debug(backup_fallback_c.size());
    memory = backup;

  // we really dont want to do this
  //print_library(&backup);
 
  // before we use rand() ...
  srand((int)time(NULL));
}

ourmove_t randomai::getourmove(move_signal sig){
  ourmove_t move;

//  debug("ai move requested!");
  if(sig == NORMAL){
    // previous move was allowed so ...
    // many moves to try
    memory = backup;

   debug("previous move ok!");

    // first we use our standard move database
    fallback_a = 0;
    fallback_b = 0;
  }
  else{
    debug("previous ai move not allowed!");

    
    if(!fallback_a && memory.size() == 0){
      debug("switching to fallback_a library");
      memory = backup_fallback_a;
      fallback_a = 1;
    }
    if(fallback_a && (memory.size() == 0) && !fallback_b){
      debug("switching to fallback_b library");
      memory = backup_fallback_b;
      fallback_b = 1;
    }    
    if(fallback_b && memory.size() == 0){
      debug("switching to fallback_c library");
      memory = backup_fallback_c;
      // 
    }
  }

  debug("remaining moves to test:");
  debug(memory.size());

  int r = rand()%memory.size();
  move = memory[r];
  memory.erase(memory.begin()+r);

  //debug("last line of getourmove()");  
  //debug("this is the chosen move:");  
  //print_move(&move);
  //debug("... and this is the last line of getourmove()");  
  return move;
}

