// Copyright (C) 2006 The Ascal Authors - see AUTHORS for more information
//  
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software 
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#include "shareddata.hpp"

#include "player.hpp"
#include "helper.h"
#include "ourmove.hpp"

#include <iostream>
#include <fstream>
#include <sstream>
#include <string>

shareddata::shareddata(){
	load_history_flag=false;
}

ourmove_t shareddata::request_move(bool color, move_signal sig){
	ourmove_t move;
	
	//playing back the history
	if(!load_history.empty()) {
		debug("loading data from saved game");
		move=load_history.front();
		load_history.pop_front();
		if(move.back()==-1)		//ignore the officer flag
			move.pop_back();
		return move;
	}
	else if(load_history_flag==1) {
		debug("load_history_flag == 1");
		load_history_flag=3;
		move.push_back(-3);
		return move;
	}
	else if(load_history_flag==3) {
	  debug("load_history_flag == 3");
	  numberai = numberai_backup;
	  debug(numberai_backup);

	  move.push_back(-8);
	  
	  load_history_flag = 0;
	  return move;
	}

  mut_requests.lock();

  request_t current_request = {1, color, sig};

  move_requests.push_back(current_request);

  if(color == WHITE){
    while(moves_white.empty()){
      move_available.wait(mut_requests);
    }
    move = moves_white.front();
    moves_white.pop_front();
  }
  if(color == BLACK){
    while(moves_black.empty()){
      move_available.wait(mut_requests);
    }
    move = moves_black.front();
    moves_black.pop_front();    
  }
  mut_requests.unlock();

  return move;
}

request_t shareddata::check_moves(){
  request_t result = {0, 0
  , NORMAL};
  mut_requests.lock();
  if(!move_requests.empty()){
    result = move_requests.front();
    if(result.sig==RESET) {
    	move_requests.pop_front();
    }
  }
  mut_requests.unlock();
  return result;
}

bool shareddata::add_move(request_t request, ourmove_t move){
  bool result = false;
  mut_requests.lock();
  if(!move_requests.empty() && request.requested){
    if(move_requests.front().color == request.color){
      move_requests.pop_front();      
      if(request.color == WHITE){
	moves_white.push_back(move);
      }
      if(request.color == BLACK){
	moves_black.push_back(move);
      }
      move_available.signal();
      result = true;
    }    
  }
  mut_requests.unlock();

  return result;
}

void shareddata::add_move_to_display(fieldvec_t move){
	if(!load_history.empty()) {
		return;
	}
	else if (load_history_flag==2) {
		load_history_flag=1;
		return;
	}

  mut_moves_to_display.lock();
  moves_to_display.push_back(move);
  //while(!moves_to_display.empty()){
  //  everything_displayed.wait(mut_moves_to_display);
  //}
  mut_moves_to_display.unlock();
}

fieldvec_t shareddata::rem_move_to_display(){
  fieldvec_t move;
  mut_moves_to_display.lock();
  move = moves_to_display.front();
  moves_to_display.pop_front();
  everything_displayed.signal();
  mut_moves_to_display.unlock();
  return move; 
}

bool shareddata::no_moves_to_display(){
  bool empty;
  mut_moves_to_display.lock();
  empty = moves_to_display.empty();
  mut_moves_to_display.unlock();
  
  return empty;
}

void shareddata::kill_logic() {
	debug("Trying to kill the logic");
	ourmove_t psi_move;
	psi_move.push_back(-1);
	request_t re={1,0,NORMAL};
	add_move(re,psi_move);
	//we send it to both players - it's better!
	//a generic way would be better but okay...
	re.color=1;
	add_move(re,psi_move);
}


void shareddata::reconfigure_players(int ai_players) {
	debug("Trying to reconfigure the players");
	ourmove_t psi_move;
	numberai= ai_players;
	if(ai_players == 0){
	debug("0 ai players");
	psi_move.push_back(-8);
	}
	if(ai_players == 1){
	debug("1 ai players");
	psi_move.push_back(-8);
	}
	if(ai_players == 2){
	debug("2 ai players");
	psi_move.push_back(-8);
	}
	request_t re={1,0,NORMAL};
	add_move(re,psi_move);
	//we send it to both players - it's better!
	//a generic way would be better but okay...
	re.color=1;
	add_move(re,psi_move);
}

void shareddata::go_back(request_t re) {
	//FIXMEmoh: fill me 
	debug("Trying to go back");
	ourmove_t psi_move;
	psi_move.push_back(-2);
		
	add_move(re,psi_move);
}

void shareddata::update_all() {
	debug("Trying to update all");
	ourmove_t psi_move;
	psi_move.push_back(-3);
	request_t re={1,0,NORMAL};
	add_move(re,psi_move);
	//we send it to both players - it's better!
	//a generic way would be better but okay...
	re.color=1;
	add_move(re,psi_move);
}

void shareddata::clean_up() {
  // all lists have to be cleared
  mut_requests.lock();
  move_requests.clear();
  moves_white.clear();
  moves_black.clear();
  mut_requests.unlock();

  mut_moves_to_display.lock();
  moves_to_display.clear();
  mut_moves_to_display.unlock();

  while( !history.empty() ) {
    history.pop_back();
  }
  while( !load_history.empty() ) {
    load_history.pop_back();
  }
  load_history_flag=false;

  // numberai stays untouched
}

void shareddata::setnumberai(int number) {
  mut_numberai.lock();
  if(number==-1){
    debug("#### backing up numberai!");
    debug(numberai);
    numberai_backup = numberai;
    numberai = 0;
  }
  else{
    numberai = number;
  }
  mut_numberai.unlock();
}

int shareddata::getnumberai() {
  return numberai;
}


void shareddata::dump_moves_in_queue(){
  mut_requests.lock();
  debug("white queue:");
  std::list<ourmove_t>::iterator it_move;
  for( it_move = moves_white.begin();
       it_move != moves_white.end(); it_move++ ) {
    debug("move in queue");
  }   

  debug("black queue:");
  for( it_move = moves_black.begin();
       it_move != moves_black.end(); it_move++ ) {
    debug("move in queue");
  }   
  mut_requests.unlock();
}

bool shareddata::history_empty() {
  return history.empty();
}

bool shareddata::save_history(std::string newfile){
  debug("saving history to file");

  std::string::size_type loc = newfile.find( '.', 0 );
  if( loc == std::string::npos ){
    newfile.append(".asc");
  }

  std::ofstream file(newfile.c_str(), std::ios::trunc);

  if(!file) return false;

  ourmove_t temp_move;
  std::ostringstream temp_stream;
  std::list<ourmove_t>::iterator it_hist;
  std::vector<int>::iterator it_move;
  for( it_hist = history.begin();
       it_hist != history.end(); it_hist++ ) {
    temp_move = *it_hist;
    for( it_move = temp_move.begin(); it_move != temp_move.end(); it_move++ ) {
      file << " " <<*it_move;
    }
    file << '\n';
  }     
  file.close();

  return true;
}

bool shareddata::open_history(std::string newfile){
  debug("loading history from file");

  std::ifstream file(newfile.c_str());

  if(!file) return false;

  ourmove_t temp_move;
  int move_number;

  load_history.clear();

  while(!file.eof()){
    if(file.peek() == '\n'){
      file.get();
      load_history.push_back(temp_move);
      temp_move.clear();
      debug("new move");
    }
    file >> move_number;
    debug(move_number);
    temp_move.push_back(move_number);
  }
  
  // debugging here
  debug("loaded history:");
  temp_move.clear();
  std::ostringstream temp_stream;
  std::list<ourmove_t>::iterator it_hist;
  std::vector<int>::iterator it_move;
  for( it_hist = history.begin();
       it_hist != history.end(); it_hist++ ) {
    temp_move = *it_hist;
    for( it_move = temp_move.begin(); it_move != temp_move.end(); it_move++ ) {
      temp_stream << *it_move << " ";
    }
    debug(temp_stream);
  }  

  file.close();
  load_history_flag=2;
  return true;
}


void shareddata::reset_player_request() {
	request_t re;
	re.color=1;
	re.sig=RESET;
	re.requested=1;
	mut_requests.lock();
		if(!move_requests.empty())
			move_requests.pop_front();
		move_requests.push_back(re);
	mut_requests.unlock();
}
